# LSB Test Execution Framework
# Tcl test module
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF with some parts
# derived from lsb-autotest written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Tcl_test;
use strict;

use Common;
use Packages;

require Tests::Test_common;
@Tcl_test::ISA = qw(Test_common);

=head ##
Class Tcl_test Inherits Test_common;
=cut ##

sub new {
    my $self  = Test_common->new();
    $self->{NAME} = "tcl";
    $self->{DISPLAYNAME} = "Tcl Tests";
    $self->{DESCRIPTION} = "LSB Application Battery Functional Tests for Tcl,"
        ." an embedded scripting language.";
    $self->{PACKAGES} .= " lsb-tcl";
    $self->{FILES} .= " tcl-tests.tar tcl8.4.11-tests.tar tcl8.5a4-tests.tar";
    $self->{SOURCES} .= " appbat test_data";
    $self->{BETA_SOURCES} .= " appbat test_data";
    $self->{SNAPSHOT_SOURCES} .= " /snapshots/appbat test_data";

    bless($self);
    return $self;
} 

sub files {
    my $self = shift;
    my $lsb_ver = shift;
    
    my $files = $self->{FILES};
    if ( $lsb_ver eq 'snapshot' ) {
        $files = "tcl-tests.tar";
    }
    elsif ( $lsb_ver eq '3.0' || $lsb_ver eq '3.1' ) {
        $files = "tcl8.4.11-tests.tar";
    }
    else {
        $files = "tcl8.5a4-tests.tar";
    }
    
    return $files;
}

sub run {
    my $self = shift;
    
	my $tcl_tests_tar_location = "";

    if ( $lsb_version eq "snapshot" ) {
        $tcl_tests_tar_location = "snapshots/appbat/tests/tcl-tests.tar";
    }
    elsif ( $lsb_version eq '3.0' || $lsb_version eq '3.1' ) {
        $tcl_tests_tar_location = "app-battery/tests/tcl8.4.11-tests.tar";
    }
    else {
        $tcl_tests_tar_location = "app-battery/tests/tcl8.5a4-tests.tar";
    }
   
    unless ( -d "/tmp/lsb-packages" ) {
        cmd("mkdir -p /tmp/lsb-packages");
    }
    cmd ("cd /tmp/lsb-packages".
         " && ln -s -f \"$package_dir/$tcl_tests_tar_location\" tcl-tests.tar"
        );

    Test_common::run($self);
}

sub journal {
    my $self = shift;
    
    store_journal_info(
        "Certification Tests/Automated Application Battery Tests/".$self->{DISPLAYNAME}
        , "tcl", "$results_dir/test_journals/tcl.tcljournal", $self);
}

1; # return value
