# LSB Test Execution Framework
# Common test module
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF with some parts
# derived from lsb-autotest written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


# Common interface class for all test objects.
    
package Test_common;
use strict;

use Common;

=head ## Class description

class Test_common{
    NAME read:name;
    DISPLAYNAME read:name;
    DESCRIPTION;
    PACKAGES read:packages;
    SOURCES read:sources;
    BETA_SOURCES read:beta_sources;
    SNAPSHOT_SOURCES read:snapshot_sources;
    FILES read:files;
    START_TIME;
    STOP_TIME;
    
    &check;
    &run;
    &cleanup;
    &is_available;
    &journal;
}

=cut ##

# Class constructor
sub new {
    my $self  = {};
    $self->{NAME} = "";
    $self->{DISPLAYNAME} = "";
    $self->{DESCRIPTION} = "";
    $self->{PACKAGES} = "";
    $self->{SOURCES} = "";
    $self->{BETA_SOURCES} = "";
    $self->{SNAPSHOT_SOURCES} = "";
    $self->{FILES} = "";
    $self->{START_TIME} = undef;
    $self->{STOP_TIME} = undef;
    $self->{FAILED} = undef;

    bless($self);
    return $self;
}

# Read methods for class fields
sub name { return shift->{NAME} };
sub displayname { return shift->{DISPLAYNAME} };
sub packages { return shift->{PACKAGES} };
sub sources { return shift->{SOURCES} };
sub beta_sources { return shift->{BETA_SOURCES} };
sub snapshot_sources { return shift->{SNAPSHOT_SOURCES} };
sub files { return shift->{FILES} };

# Abstract class methods.
# Each test shall override necessary methods

# Do some initial environment checking.
sub check {
    my $self = shift;
    
    return 1;
}

# Run the test
sub run
{
    my $self = shift;
    
    my $command = "$autotest_cmd ".$self->name;
    debug_inform("% ".$command);
    my $result = system ($command) >> 8;
    
    return $result;
}

# Clean up. This method will be called even if the test fails
sub cleanup {
    my $self = shift;
}

# Check the test is available
sub is_available {
    my $self = shift;
    return 1;
}

# Store report files and create the record in journal
sub journal {
    my $self = shift;
}

1; # return value
