# LSB Test Execution Framework
# Common X11 test module
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF with some parts
# derived from lsb-autotest written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


# Common interface class for X11-oriented tests
    
package X11_common;
use strict;

use Tests::Test_common;
@X11_common::ISA = qw(Test_common);

use Common;

=head ##
Class X11_common Inherits Test_common {
    overrides &check;
}
=cut ##

sub new {
    my $self  = Test_common->new();
    bless($self);
    $self->{SNAPSHOT_SOURCES} .= ' /snapshots/lsb-xvfb';
    return $self;
}

sub check_font_dir {
    my $x_font_dir = shift;
    $x_font_dir =~ s/(^\s+|\s+$|\n)//g; #trim
    
    return 0 unless ( -d $x_font_dir );
    return 0 unless ( -d "$x_font_dir/misc/" );
    return 0 unless ( -d "$x_font_dir/100dpi/" );
    return 0 unless ( -d "$x_font_dir/75dpi/" );
    return 0 unless ( -f "$x_font_dir/100dpi/timR24.pcf.gz" );
    
    debug_inform("FontCheck has satisfied with $x_font_dir");
    
    return 1;
}

# Check for 100dpi fonts needed for X-oriented tests.
# Returns 0 on error and 1 on success
sub check_fonts {
    my $self = shift;
    
    my $x_font_dir = "";
    
    # Try to grep font path from /etc/X11/xorg.conf
    if ( -f "/etc/X11/xorg.conf") {
        $x_font_dir = capture('grep -e "FontPath" /etc/X11/xorg.conf'
                .' | grep 75dpi | head -n1' );
        $x_font_dir =~ s/^\s+FontPath\s+\"?//;
        $x_font_dir =~ s/\/75dpi.*//;
        return 1 if check_font_dir($x_font_dir);
    }
    
    # Else, try to grep font path from /etc/X11/fs/config
    if ( -f "/etc/X11/fs/config") {
        $x_font_dir = capture('cat /etc/X11/fs/config | grep 75dpi'
                .' | head -n1'
            );
        $x_font_dir =~ s/^\s+\"?//;
        $x_font_dir =~ s/\"?,?//;
        $x_font_dir =~ s/\/75dpi.*//;
        return 1 if check_font_dir($x_font_dir);
    }
    
    return 1 if check_font_dir("/usr/share/fonts");
    return 1 if check_font_dir("/usr/share/fonts/X11");
    return 1 if check_font_dir("/usr/share/X11/fonts");
    return 1 if check_font_dir("/usr/X11R7/lib/X11/fonts");
    return 1 if check_font_dir("/usr/X11R6/lib/X11/fonts");

    # Can't find anything
    complain("Could not find the X11 fonts directory");
    return 0;
}

# Returns 0 on error and 1 on success
sub check {
    my $self = shift;
    my $failed = 0;
    
    # Check for additional tools and packages.

    # The X virtual framebuffer server is used for testing.
    if ( $lsb_version eq '3.0' || $lsb_version eq '3.1' ) {
        unless (does_shell_know ("Xvfb")) {
            complain ("The ".$self->name." test can't be run."
                ." Please install the Xvfb server first."
            );
            $failed = 1;
        }
        $failed = 1 if !check_fonts($self);
    }
    
    return 0 if $failed;
    
    return Test_common::check($self);
}

sub run {
    my $self = shift;
    
    cmd ("killall -q Xvfb");
    
    return 1 if !$self->check();
    
    return Test_common::run($self);
}

sub cleanup {
    my $self = shift;
    
    cmd ("killall -q Xvfb");
    
    return Test_common::cleanup($self);
}

1; # return value
