# LSB Test Execution Framework
# Xts5 test module
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF with some parts
# derived from lsb-autotest written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Xts5_test;
use strict;

use Common;
use Packages;

use Tests::X11::X11_common;
@Xts5_test::ISA = qw(X11_common);

=head ##
Class Xts5_test Inherits X11_common
=cut ##

sub new {
    my $self  = X11_common->new();
    $self->{NAME} = "xts5";
    $self->{DISPLAYNAME} = "X11 Tests (XTS5)";
    $self->{DESCRIPTION} = "This is a binary version of the"
        ." LSB X Window System test suite. These tests are derived from"
        ." the XTS5 test suites.";
    $self->{PACKAGES} .= " lsb-tet3 lsb-xvfb lsb-test-xts5";
    $self->{SOURCES} .= " runtime";
    $self->{BETA_SOURCES} .= " runtime";
    $self->{SNAPSHOT_SOURCES} .= " /snapshots/tet-harness /snapshots/xts5-test";
    
    bless($self);
    return $self;
}

sub packages {
    my $self = shift;
    my $lsb_ver = shift;

    if ( !defined($lsb_ver) ) { return $self->{PACKAGES}; }

    if ( $lsb_ver eq "3.0" || $lsb_ver eq "3.1" ) {
        my $packages = $self->{PACKAGES};
        $packages =~ s/ lsb-xvfb / /g;
        return $packages;
    }
    return $self->{PACKAGES};    
}

sub check {
    my $self = shift;
    my $failed = 0;
    
    # Depends on xlsfonts command
    unless (does_shell_know ("xlsfonts")) {
        complain ("The ".$self->name." test can't be run."
            ." Can't find the 'xlsfonts' command."
        );
        $failed = 1;
    }
    
    return 0 if $failed;
    
    return X11_common::check($self);
}

sub run {
    my $self = shift;
    
    return X11_common::run($self);
}

sub journal {
    my $self = shift;
    
    if ( $lsb_version eq '3.0' || $lsb_version eq '3.1' ) {
        # Was unofficial until LSB 3.2
        store_journal_info(
            "Unofficial Tests/Runtime Interface Tests/".$self->{DISPLAYNAME}
            , "xts5_test", "$results_dir/test_journals/xts5_test.journal", $self);
    } else {
        # Official test since LSB 3.2
        store_journal_info(
            "Certification Tests/Runtime Interface Tests/".$self->{DISPLAYNAME}
            , "xts5_test", "$results_dir/test_journals/xts5_test.journal", $self);
    }
}

sub cleanup {
    my $self = shift;
    
    X11_common::cleanup($self);
}

1; # return value
