#!/bin/bash

port=$1
if [ -z "$port" ]; then
	port="8888"
else
	confirm=1
fi

SUDO_CMD=$2

if [ -z "$2" ]; then
	if lsb_release -ds | grep -i "ubuntu" >/dev/null 2>&1; then
		SUDO_CMD="sudo su -c"
	else
		SUDO_CMD="su -c"
	fi
else 
	confirm=1
fi

echo "    The port $port will be used by the LDTK Manager web-UI server."
if (( $UID != 0 )); then
	echo "    The command '$SUDO_CMD' will be used to gain root access."
	echo "    If you want to change this, run $0 <port> <sudo command>"
else
	echo "    If you want to change this, run $0 <port>"
fi

echo

if [ ! -z "$confirm" ] ; then
	echo -n "Continue?"
	read answer
	answer=`echo ${answer} | cut -c1`
	if [ "${answer}" '!=' "y" -a "${answer}" '!=' "Y" ]; then
	    exit 4
	fi
fi

ps -A | grep '\bdtk-server.pl\b' >/dev/null 2>&1
if (( $? ))
then
    cd `dirname $0`
    if (( $UID != 0 )); then
        echo "    Please type the appropriate password if prompted."
        echo -n "    "
    	$SUDO_CMD "./server_start.sh $port"
		echo 
	else
    	./server_start.sh $port
    fi
    
    if (( $? )) ; then 
        echo "    Couldn't start server."
        exit $?
    fi
else
    echo "    Server is already running."
fi

sleep 1

# Try to open start page in a browser
echo ""
echo "    The start page should be opened in a browser shortly."
echo "    If it doesn't open, you can load it at http://`uname -n`:$port"

browser_not_found=5

# First, try the Portland's xdg-open util.
if which xdg-open >/dev/null 2>&1
then
	TMP_ERR_FILE="/tmp/atk-xdg-err.$$"
	xdg-open "http://127.0.0.1:$port" >$TMP_ERR_FILE 2>&1
	#An exit code of 0 indicates success.
	#1 Error in command line syntax. 
	#2 One of the files passed on the command line did not exist. 
	#3 A required tool could not be found. 
	#4 The action failed.
	browser_not_found=$?
	if (( $browser_not_found )); then cat $TMP_ERR_FILE; fi
	unlink $TMP_ERR_FILE >/dev/null 2>&1
else
	# Try to find a browser by it's name.
	for br_name in "opera" "/usr/bin/mozilla-firefox" "firefox" "konqueror" "epiphany" "galeon" "mozilla"
	do
		if which $br_name >/dev/null 2>&1 
		then
			`which $br_name` "http://127.0.0.1:$port" >/dev/null 2>&1 &
			browser_not_found=0
			break
		fi
	done
fi

if  (( $browser_not_found ))
then
	echo 
	echo "    Could not find a web-browser."
	echo "    Go to 'http://`uname -n`:$port' to configure and launch tests."
fi
