<?
/*
# LSB Test Execution Framework
# Standalone web-page displaying result directory (auto_dtk_results.php)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

// SETTINGS
$result_dir = '/var/opt/lsb/test/manager/results';
$www_root = 'http://yourhost.org/results';

// FUNCTIONS

// Escapes data for safe printing in the HTML code.
function html($text)
{
	return htmlspecialchars($text, ENT_QUOTES);
}

// Read the result dir
$handle = opendir($result_dir) or die('Can\'t open directory with results.');
$dirs = array();
while ( false !== ( $file = readdir($handle) )  ) {
    if ($file == "." || $file == "..")  continue;
    if ( !is_dir("$result_dir/$file") ) continue;
    
    // Read INFO file
    $info_lines = @file ( "$result_dir/$file/INFO" );
    if ( $info_lines === False ) continue;
    $summary = '';
    while ( $line = each($info_lines) ) {
        $line = $line['value'];
        if ( $line == "\n" ) break; // empty line preceedes comment.
        if ( $line == "Verdict: Success\n" ) {
            $summary .= '<span style="color:#007000"><b>'.html($line)
                .'</b></span><br />';
        }
        elseif ( $line == "Verdict: Failed\n" ) {
            $summary .= '<span style="color:#C00000"><b>'.html($line)
                .'</b></span><br />';
        }
        else
            $summary .= html($line).'<br />';
    }
    // Rest lines is a comment
    $comment = '';
    while ( $line = each($info_lines) ) {
        $line = $line['value'];
        $comment .= $line;
    }
    
    if ( preg_match( '/(\w+)-(\w+)-(\d+)-(\d+)-(\d+)-(\d+)h-(\d+)m-(\d+)s/'
            , $file, $matches )  ) {
        $arch = $matches[1];
        $host = $matches[2];
        // Date/Time
        $dt['y'] = $matches[3];
        $dt['m'] = $matches[4];
        $dt['d'] = $matches[5];
        $dt['h'] = $matches[6];
        $dt['mi'] = $matches[7];
        $dt['s'] = $matches[8];
    }
    
    $result = array (
        'name' => $file,
        'summary' => $summary,
        'comment' => $comment,
        'arch' => $arch,
        'host' => $host,
        'dt' => $dt,
    );
    $results[] = $result;
}
closedir($handle);

if ( $results ) rsort($results);

?>
<?/* Page header */?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">

<head>
	<title>Test Results</title>
    <style type="text/css">
        table {
            border: 1px solid black;
            border-collapse: collapse;
        }
        table td, table th {
            border: 1px solid black;
            padding: 8px;
            white-space: nowrap;
        }
        a {
    		color: #00007f;
    		text-decoration: none;
    	}
    	a:hover {
    		color: #00007f;
    		text-decoration: underline;
    	}
    </style>
</head>

<body>
<?/* Page header end */?>

<h1>Test Results</h1>

<?
if ( $results ) {
    ?><table>
    <tr>
    <td><b>Detailed Report</b></td>
    <td><b>Summary</b></td>
    <td><b>Comment</b></td>
    </tr>
    <?
    foreach ( $results as $result ) {
        if ( $result['host'] ) {
            $link_text = '<b>'.html($result['host']).'</b>'
                .' ('.html($result['arch']).')'
                .' '.$result['dt']['h'].':'.$result['dt']['mi'].':'.$result['dt']['s']
                .' '.$result['dt']['d'].'/'.$result['dt']['m'].'/'.$result['dt']['y']
                ;
        } else {
            $link_text = html($result['name']);
        }
        ?>
        <tr><td>
        <a href="<?=$www_root?>/<?=html($result['name'])?>/test_log.html"
           style='font-size:16pt'><?=$link_text?></a>
        </td><td>
        <code><?=$result['summary'];?></code>
        </td><td>
        <code><?=$result['comment'];?></code>
        </td></tr>
        <?
    }
    ?></table><?
}
else {
// Empty dir
    ?><h2>Empty</h2><?
}
?>

<?/* Page footer */?>

</body>
</html>
<?/* Page footer end */?>
