# LSB Test Execution Framework
# Module for Generating Lists of Tests (BuildList.pm)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package BuildList;
use strict;
require Exporter;

@BuildList::ISA = "Exporter";
@BuildList::EXPORT = qw(@lsb_versions_list %default_tests_options &generate_modlist &generate_js_mod_cat_list &generate_js_check_list &generate_js_arch_mod &generate_js_mod_list);

our @lsb_versions_list = qw/3.2 3.1 3.0/;
our %default_tests_options = ("cmdchk" => 3, "libchk" => 3, "desktop" => 3);

# %(arch-lsb => %(category|all => @mod_list))
my %check_list = ();
my %mod_cat_list = ();

# All the possible tests for all architectures and LSB versions.
# The list will be filtered by real data obtained from lsb-tef.pl.
my %tests_tree = (
	"3.0" => [
		{"name" => "cert", "dispname" => "Certification Tests", "level2" => [
			{"name" => "cmdtests", "dispname" => "Command Tests", "level3" => [
				{"name" => "cmdchk", "dispname" => "cmdchk"}
			]},
			{"name" => "sitests", "dispname" => "Static Interface Tests", "level3" => [
				{"name" => "libchk", "dispname" => "libchk"}
			]},
			{"name" => "ritests", "dispname" => "Runtime Interface Tests", "level3" => [
				{"name" => "core", "dispname" => "core"},
				{"name" => "libstdcpp", "dispname" => "libstdcpp"},
				{"name" => "vsw4", "dispname" => "vsw4"}
			]},
			{"name" => "abtests", "dispname" => "Automated Application Battery Tests", "level3" => [
				{"name" => "apache", "dispname" => "apache"},
				{"name" => "expect", "dispname" => "expect"},
				{"name" => "groff", "dispname" => "groff"},
				{"name" => "lynx", "dispname" => "lynx"},
				{"name" => "python-app", "dispname" => "python-app"},
				{"name" => "raptor", "dispname" => "raptor"},
				{"name" => "rsync", "dispname" => "rsync"},
				{"name" => "samba", "dispname" => "samba"},
				{"name" => "tcl", "dispname" => "tcl"},
				{"name" => "xpdf", "dispname" => "xpdf"}
			]},
			{"name" => "mantests", "dispname" => "Manual Tests", "level3" => [
				{"name" => "manappbat", "dispname" => "manappbat"}
			]}
		]},
		{"name" => "unoff", "dispname" => "Unofficial Tests", "level2" => [
			{"name" => "uritests", "dispname" => "Runtime Interface Tests", "level3" => [
				{"name" => "xts5", "dispname" => "xts5"},
				{"name" => "qt3-azov", "dispname" => "qt3-azov"},
				{"name" => "qt4-azov", "dispname" => "qt4-azov"},
				{"name" => "olver", "dispname" => "OLVER"}
			]}
		]}
	],
	"3.1" => [
		{"name" => "cert", "dispname" => "Certification Tests", "level2" => [
			{"name" => "cmdtests", "dispname" => "Command Tests", "level3" => [
				{"name" => "cmdchk", "dispname" => "cmdchk"}
			]},
			{"name" => "sitests", "dispname" => "Static Interface Tests", "level3" => [
				{"name" => "libchk", "dispname" => "libchk"}
			]},
			{"name" => "ritests", "dispname" => "Runtime Interface Tests", "level3" => [
				{"name" => "core", "dispname" => "core"},
				{"name" => "desktop", "dispname" => "desktop"},
				{"name" => "libstdcpp", "dispname" => "libstdcpp"},
				{"name" => "vsw4", "dispname" => "vsw4"}
			]},
			{"name" => "abtests", "dispname" => "Automated Application Battery Tests", "level3" => [
				{"name" => "apache", "dispname" => "apache"},
				{"name" => "expect", "dispname" => "expect"},
				{"name" => "groff", "dispname" => "groff"},
				{"name" => "lynx", "dispname" => "lynx"},
				{"name" => "python-app", "dispname" => "python-app"},
				{"name" => "raptor", "dispname" => "raptor"},
				{"name" => "rsync", "dispname" => "rsync"},
				{"name" => "samba", "dispname" => "samba"},
				{"name" => "tcl", "dispname" => "tcl"},
				{"name" => "xpdf", "dispname" => "xpdf"}
			]},
			{"name" => "mantests", "dispname" => "Manual Tests", "level3" => [
				{"name" => "manappbat", "dispname" => "manappbat"}
			]}
		]},
		{"name" => "unoff", "dispname" => "Unofficial Tests", "level2" => [
			{"name" => "uritests", "dispname" => "Runtime Interface Tests", "level3" => [
				{"name" => "xts5", "dispname" => "xts5"},
				{"name" => "qt3-azov", "dispname" => "qt3-azov"},
				{"name" => "qt4-azov", "dispname" => "qt4-azov"},
				{"name" => "olver", "dispname" => "OLVER"}
			]}
		]}
	],
	"3.2" => [
		{"name" => "cert", "dispname" => "Certification Tests", "level2" => [
			{"name" => "cmdtests", "dispname" => "Command Tests", "level3" => [
				{"name" => "cmdchk", "dispname" => "cmdchk"}
			]},
			{"name" => "sitests", "dispname" => "Static Interface Tests", "level3" => [
				{"name" => "libchk", "dispname" => "libchk"}
			]},
			{"name" => "ritests", "dispname" => "Runtime Interface Tests", "level3" => [
				{"name" => "core", "dispname" => "core"},
				{"name" => "desktop", "dispname" => "desktop"},
				{"name" => "desktop-t2c", "dispname" => "desktop-t2c"},
				{"name" => "libstdcpp", "dispname" => "libstdcpp"},
				{"name" => "perl", "dispname" => "perl"},
				{"name" => "printing", "dispname" => "printing"},
				{"name" => "python", "dispname" => "python"},
				{"name" => "xts5", "dispname" => "xts5"},
				{"name" => "qt3-azov", "dispname" => "qt3-azov"},
				{"name" => "qt4-azov", "dispname" => "qt4-azov"},
				{"name" => "olver", "dispname" => "OLVER"}
			]},
			{"name" => "abtests", "dispname" => "Automated Application Battery Tests", "level3" => [
				{"name" => "apache", "dispname" => "apache"},
				{"name" => "expect", "dispname" => "expect"},
				{"name" => "groff", "dispname" => "groff"},
				{"name" => "lynx", "dispname" => "lynx"},
				{"name" => "python-app", "dispname" => "python-app"},
				{"name" => "raptor", "dispname" => "raptor"},
				{"name" => "rsync", "dispname" => "rsync"},
				{"name" => "samba", "dispname" => "samba"},
				{"name" => "tcl", "dispname" => "tcl"},
				{"name" => "xpdf", "dispname" => "xpdf"}
			]},
			{"name" => "mantests", "dispname" => "Manual Tests", "level3" => [
				{"name" => "manappbat", "dispname" => "manappbat"}
			]}
		]}
	]
);

my %tests_options_tree = (
	"3.1" => {
		"cmdchk" => {
			2 => "Core &amp; C++",
			3 => "Core &amp; C++ &amp; Desktop"
		},
		"libchk" => {
			2 => "Core &amp; C++",
			3 => "Core &amp; C++ &amp; Desktop",
			4 => "Core &amp; C++ &amp; Desktop &amp; Qt4 (<i>optional</i>)"
		},
		"desktop" => {
			3 => "Desktop only",
			4 => "Desktop &amp; Qt4 (<i>optional</i>)"
		}
	},
	"3.2" => {
		"cmdchk" => {
			2 => "Core &amp; C++",
			3 => "Core &amp; C++ &amp; Desktop"
		},
		"libchk" => {
			2 => "Core &amp; C++",
			3 => "Core &amp; C++ &amp; Desktop"
		}
	}
);

sub build_test_suite_row($$$$$) {
	my $res = "";
	my ($lsbv_arch, $bkgd, $arch, $mod, $lsb_v) = @_;
	my $modinfo = $lsbv_arch->{$lsb_v}->{$arch}->{$mod};
	my $stage = $modinfo->{"STAGES"};
	$res .= "        <tr class=\"list_item$bkgd\">\n          <td><label><input type=\"checkbox\" name=\"mod-$arch-$lsb_v-$mod\" id=\"mod-$arch-$lsb_v-$mod\" onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\" />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;".$modinfo->{"DISPLAY_NAME"}."</label></td>\n";

	my $checked = " checked=\"checked\"";
	foreach (qw(p a b)) {
		my $state;
		my $id;
		if ($stage->{$_}) {
			my $state = "$checked disabled=\"disabled\" onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\"";
			$checked = "";
			my $id = "-$_";
			$res .= "          <td><label><input type=\"radio\" name=\"modstage-$arch-$lsb_v-$mod\" id=\"modstage-$arch-$lsb_v-$mod$id\" value=\"$_\"$state /> ".$stage->{$_}->{"VERSION"}."</label></td>\n";
		}
		else {
			my $state = " disabled=\"disabled\"";
			my $id = "-$_-";
			$res .= "          <td><label><input type=\"radio\" name=\"modstage-$arch-$lsb_v-$mod\" id=\"modstage-$arch-$lsb_v-$mod$id\" value=\"$_\"$state /></label></td>\n";
		}
	}
	$res .= "        </tr>\n";

	if ($tests_options_tree{$lsb_v}) {
		my $modopt = $tests_options_tree{$lsb_v}->{$mod};
		if ($modopt) {
			foreach my $option (sort keys %$modopt) {
				my $checked = (($default_tests_options{$mod} == $option) ? " checked=\"checked\"" : "");
				$res .= "        <tr class=\"list_item$bkgd\">\n          <td colspan=\"4\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<label><input type=\"radio\" name=\"modopt-$arch-$lsb_v-$mod\" id=\"modopt-$arch-$lsb_v-$mod-$option\" value=\"$option\" disabled=\"disabled\"$checked onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\" />&nbsp;".$modopt->{$option}."</label></td>\n        </tr>\n";
			}
		}
	}

	return $res;
}

sub generate_modlist($) {
	my $res = "";
	my ($lsbv_arch) = @_;
	foreach my $lsb_v (@lsb_versions_list) {
		foreach my $arch (sort keys %{$lsbv_arch->{$lsb_v}}) {
			$res .= "      <table id=\"arch-$arch-$lsb_v\" cellpadding=\"0\" cellspacing=\"0\" style=\"border: solid 4px white; width: 100%; display: none;\">\n";
			my $count;
			my %to_list = ();
			foreach my $mod (keys %{$lsbv_arch->{$lsb_v}->{$arch}}) {
				$to_list{$mod} = 1;
			}
			$res .= <<DATA;
        <tr class="list_head">
          <td style="width: 14.5em;"><input type="checkbox" name="check-all-$arch-$lsb_v" id="check-all-$arch-$lsb_v" onclick="javascript:un_check_all('all');" /></td>
          <td style="width: 10em;"><a href="" onclick="javascript:return select_tests_column('p');" class="clickable" title="Select all local test suites"><b>Localhost</b></a></td>
          <td style="width: 10em;"><a href="" onclick="javascript:return select_tests_column('a');" class="clickable" title="Select latest releases of test suites"><b>FTP</b></a></td>
          <td style="width: 10em;"><a href="" onclick="javascript:return select_tests_column('b');" class="clickable" title="Select latest beta-versions of test suites"><b>FTP (beta)</b></a></td>
        </tr>
DATA

			$check_list{"$arch-$lsb_v"} = {"all" => []};
			$mod_cat_list{"$arch-$lsb_v"} = {"all" => []};

			foreach my $level1_cat (@{$tests_tree{$lsb_v}}) {
				my $cat_name1 = $level1_cat->{"name"};
				my $disp_name1 = $level1_cat->{"dispname"};
				$mod_cat_list{"$arch-$lsb_v"}->{$cat_name1} = [];
				$check_list{"$arch-$lsb_v"}->{$cat_name1} = [];
		  		push @{$check_list{"$arch-$lsb_v"}->{"all"}}, $cat_name1;
				$res .= <<DATA;
        <tr class="list_cat">
          <td colspan="4"><b><label><input type="checkbox" name="check-$cat_name1-$arch-$lsb_v" id="check-$cat_name1-$arch-$lsb_v" onclick="javascript:un_check_all('$cat_name1');" />&nbsp;$disp_name1</label>&nbsp;&nbsp;&nbsp;<a href="tests_help.pl#$cat_name1" onclick="javascript:show_hide_tooltip('$cat_name1', event); return false;"><img src="images/question.png" width="16" height="17" alt="[?]" /></a></b></td>
        </tr>
DATA
				foreach my $level2_cat (@{$level1_cat->{"level2"}}) {
					my $cat_name2 = $level2_cat->{"name"};
					my $disp_name2 = $level2_cat->{"dispname"};
					$mod_cat_list{"$arch-$lsb_v"}->{$cat_name2} = [];
					$check_list{"$arch-$lsb_v"}->{$cat_name2} = [];
			  		push @{$check_list{"$arch-$lsb_v"}->{"all"}}, $cat_name2;
			  		push @{$check_list{"$arch-$lsb_v"}->{$cat_name1}}, $cat_name2;
					$res .= <<DATA;
        <tr class="list_subcat">
          <td colspan="4"><b><label><input type="checkbox" name="check-$cat_name2-$arch-$lsb_v" id="check-$cat_name2-$arch-$lsb_v" onclick="javascript:un_check_all('$cat_name2');" />&nbsp;&nbsp;&nbsp;$disp_name2</label>&nbsp;&nbsp;&nbsp;<a href="tests_help.pl#$cat_name2" onclick="javascript:show_hide_tooltip('$cat_name2', event); return false;"><img src="images/question.png" width="16" height="17" alt="[?]" /></a></b></td>
        </tr>
DATA
					$count = 0;
				  	foreach my $level3_mod (@{$level2_cat->{"level3"}}) {
				  		my $mod = $level3_mod->{"name"};
						if ($to_list{$mod}) {
					  		push @{$mod_cat_list{"$arch-$lsb_v"}->{"all"}}, $mod;
					  		push @{$mod_cat_list{"$arch-$lsb_v"}->{$cat_name1}}, $mod;
					  		push @{$mod_cat_list{"$arch-$lsb_v"}->{$cat_name2}}, $mod;
							$count++;
							$res .= build_test_suite_row($lsbv_arch, ($count % 2), $arch, $mod, $lsb_v);
							delete $to_list{$mod};
						}
					}
				}
			}
			$res .= "      </table>\n";
		}
	}
	return $res;
}

sub generate_js_mod_cat_list($) {
	my ($lsbv_arch) = @_;
	my $res = "mod_cat_list = new Array();\n";
	foreach my $lsb_v (@lsb_versions_list) {
		foreach my $arch (sort keys %{$lsbv_arch->{$lsb_v}}) {
			$res .= "mod_cat_list['$arch-$lsb_v'] = new Array();\n";
			foreach (sort keys %{$mod_cat_list{"$arch-$lsb_v"}}) {
				$res .= "mod_cat_list['$arch-$lsb_v']['$_'] = new Array('".join("', '", @{$mod_cat_list{"$arch-$lsb_v"}->{$_}})."');\n";
			}
		}
	}
	return $res;
}

sub generate_js_check_list($) {
	my ($lsbv_arch) = @_;
	my $res = "check_list = new Array();\n";
	foreach my $lsb_v (@lsb_versions_list) {
		foreach my $arch (sort keys %{$lsbv_arch->{$lsb_v}}) {
			$res .= "check_list['$arch-$lsb_v'] = new Array();\n";
			foreach (sort keys %{$check_list{"$arch-$lsb_v"}}) {
				$res .= "check_list['$arch-$lsb_v']['$_'] = new Array(".(scalar(@{$check_list{"$arch-$lsb_v"}->{$_}}) > 0 ? "'".join("', '", @{$check_list{"$arch-$lsb_v"}->{$_}})."'" : "").");\n";
			}
		}
	}
	return $res;
}

sub generate_js_arch_mod($) {
	my ($lsbv_arch) = @_;
	my $res = "arch_mod = new Array();\n";
	foreach my $lsb_v (@lsb_versions_list) {
		foreach my $arch (sort keys %{$lsbv_arch->{$lsb_v}}) {
			my $mods = $lsbv_arch->{$lsb_v}->{$arch};
			$res .= "arch_mod['$arch-$lsb_v'] = new Array();\n";
			foreach my $mod (sort keys %$mods) {
				my $stage = $mods->{$mod}->{"STAGES"};
				$res .= "arch_mod['$arch-$lsb_v']['$mod'] = new Array();\n";
				if ($stage->{"p"}) {
					$res .= "arch_mod['$arch-$lsb_v']['$mod']['PRESENT_VERS'] = '".$stage->{"p"}->{"VERSION"}."';\n";
					$res .= "arch_mod['$arch-$lsb_v']['$mod']['PRESENT_FILES'] = new Array('".join("', '", @{$stage->{"p"}->{"FILES"}})."');\n";
				}
				if ($stage->{"a"}) {
					$res .= "arch_mod['$arch-$lsb_v']['$mod']['AVAIL_VERS'] = '".$stage->{"a"}->{"VERSION"}."';\n";
					$res .= "arch_mod['$arch-$lsb_v']['$mod']['AVAIL_FILES'] = new Array('".join("', '", @{$stage->{"a"}->{"FILES"}})."');\n";
				}
				if ($stage->{"b"}) {
					$res .= "arch_mod['$arch-$lsb_v']['$mod']['BETA_VERS'] = '".$stage->{"b"}->{"VERSION"}."';\n";
					$res .= "arch_mod['$arch-$lsb_v']['$mod']['BETA_FILES'] = new Array('".join("', '", @{$stage->{"b"}->{"FILES"}})."');\n";
				}
			}
		}
	}
	return $res;
}

sub generate_js_mod_list($) {
	my ($lsbv_arch) = @_;
	my %mod_list = ();
	foreach my $lsb_v (@lsb_versions_list) {
		foreach my $arch (sort keys %{$lsbv_arch->{$lsb_v}}) {
			foreach my $mod (sort keys %{$lsbv_arch->{$lsb_v}->{$arch}}) {
				$mod =~ s/\+/p/g;
				$mod_list{$mod} = 1;
			}
		}
	}
	return "mod_list = new Array('".join("', '", sort keys %mod_list)."');\n";
}

1;
