#!/usr/bin/perl -w

# LSB Test Execution Framework
# AJAX Server Module (ajax_srv.pl)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use BuildList;
use Fcntl ':seek';
use Fcntl ':flock';

autoflush_on();

my $error_text = "";
my $data = "";

my $output_xml = <<DATA;
HTTP/1.0 200 OK
Content-type: text/xml

<?xml version="1.0" encoding="UTF-8"?>
DATA

sub send_reply() {
	if ($error_text) {
		$error_text = "<ul><li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$error_text</font></li></ul>";
		$error_text =~ s/&/&amp;/g;
		$error_text =~ s/</&lt;/g;
		$error_text =~ s/>/&gt;/g;
		print $output_xml."<root>\n<error>".$error_text."</error>\n</root>";
	}
	else {
		print $output_xml."<root>\n".$data."</root>";
	}
}

sub list_dir() {
	if (opendir(DIR, $SERVER_PARAM{"APP_DATA"}."/profiles")) {
		$data .= "<profiles>\n";
		foreach (sort(grep(!/^[~\.]/, readdir(DIR)))) {
			$data .= "<name>$_</name>\n";
		}
		$data .= "</profiles>\n";
		closedir(DIR);
	}
	else {
		$error_text = "Could not obtain directory list!<br />($!)";
	}
}

sub get_file_id($) {
	my ($fpath) = @_;
	my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($fpath);
	return "$dev-$ino";
}

sub process_output($) {
	return 0 if ($_[0] eq "");
	my $newln_pos = rindex($_[0], "\n");
	my $res_size = length($_[0]);
	if ($newln_pos != -1) {
		# Convert Windows-style line endings to Unix-style
		$_[0] =~ s/\r\n/\n/g;
		my @lines = split(m/^/m, $_[0]);
		for (my $i = 1; $i < scalar(@lines); $i++) {
			# Make every ^M control character (\x0d) erase all from the beginning of line
			while (($lines[$i] =~ s/^[^\x0d]*\x0d//sg) > 0) {}
			# Make every ^H control character (\x08) erase the previous character (if present)
			while (($lines[$i] =~ s/[^\x08]\x08//sg) > 0) {}
			# Remove remaining ^H characters in the beginning of line (if there were some excessive)
			$lines[$i] =~ s/^\x08+//s;
		}
		$_[0] = join("", @lines);
	}
	$_[0] =~ s/[\x00-\x08\x0b-\x0c\x0e-\x1f]/./g;           # Remaining ^H characters replaced with dots (XML parser cannot stand them)
	$_[0] =~ s/]]>/]]>]]&gt;<![CDATA[/g;                    # Escape ]]> that would close CDATA block otherwise
	$_[0] =~ s/\x0d/]]>&#13;<![CDATA[/g;                    # Escape remaining ^M characters - they come to browser as ^J
	return $res_size;
}

if (!$_GET{"action"}) {
	$error_text = "Incorrect call parameters!";
}
elsif ($_GET{"action"} eq "delete_profile") {               # Delete profile
	if ($_GET{"profile"}) {
		if (!-f $SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"}) {
			$error_text = "File not found:<br />".$SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"};
		}
		else {
			unlink($SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"});
			list_dir();
		}
	}
	else {
		$error_text = "No profile name specified!";
	}
}
elsif ($_GET{"action"} eq "save_profile") {                 # Save profile
	if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
		system("mkdir -p ".$SERVER_PARAM{"APP_DATA"}."/profiles");
		if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
			$error_text = "Could not create profile directory:<br />".$SERVER_PARAM{"APP_DATA"}."/profiles";
		}
	}
	if (!$error_text) {
		if (!$_POST{"edit_profile_name"}) {
			$error_text = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/<>\|\?\*]/) {
			$error_text = "Invalid profile name: ".$_POST{"edit_profile_name"};
		}
	}
	if (!$error_text) {
		my $full_save_name = $SERVER_PARAM{"APP_DATA"}."/profiles/".$_POST{"edit_profile_name"};
		if (open(FILE, ">$full_save_name")) {
			my $architecture = ($_POST{"architecture"} or guess_platform(`uname -m`));
			my $lsb_version  = ($_POST{"lsb_version"} or "3.2");
			my $lsb_profile  = ($_POST{"lsb_profile"} or "desktop");
			print FILE "CERTIFY:         ".$_POST{"certify"}."\n"      if ($_POST{"certify"});
			print FILE "NAME:            ".$_POST{"tester_name"}."\n"  if ($_POST{"tester_name"});
			print FILE "ORGANIZATION:    ".$_POST{"organization"}."\n" if ($_POST{"organization"});
			if ($_POST{"tester_email"}) {
				print FILE "EMAIL:           ".$_POST{"tester_email"}."\n";
				print FILE "SEND_EMAIL:      ".(defined($_POST{"send_email_report"}) ? "1" : "0")."\n";
			}

			print FILE "VERBOSE_LEVEL:   ".($_POST{"verbose_level"} ? $_POST{"verbose_level"} : "1")."\n";
			print FILE "KEEP_PACKAGES:   ".(defined($_POST{"keep_packages"}) ? "1" : "0")."\n";
			print FILE "SHORT_SCENARIOS: ".(defined($_POST{"short_scen"}) ? "1" : "0")."\n";
			print FILE "ARCHITECTURE:    $architecture\n";
			print FILE "USE_INTERNET:    ".(defined($_POST{"use_internet"}) ? "1" : "0")."\n";
			print FILE "LSB_VERSION:     $lsb_version\n";
			print FILE "LSB_PROFILE:     $lsb_profile\n";

			if ($_POST{"comments"}) {
				$_POST{"comments"} =~ s/\r\n/\n/g;
				$_POST{"comments"} =~ s/&/&amp;/g;
				$_POST{"comments"} =~ s/</&lt;/g;
				$_POST{"comments"} =~ s/>/&gt;/g;
				my @comments_lines = split(/\n/, $_POST{"comments"});
				foreach (@comments_lines) {
					print FILE "COMMENTS: $_\n";
				}
			}

			my $manual_present = 0;
			my $automatic_present = 0;
			my $test_suites = '';
			foreach (sort grep /mod-$architecture-$lsb_version-/, keys %_POST) {
				my $tmp = $_;
				$tmp =~ s/^mod-/modstage-/;
				s/^mod-$architecture-$lsb_version-//;
				$test_suites .= "$_ ".$_POST{$tmp}."\n";
				if ($_ eq 'manappbat') {
					$manual_present = 1;
				}
				else {
					$automatic_present = 1;
				}
				$tmp =~ s/^modstage-/modopt-/;
				if ($_POST{$tmp}) {
					$test_suites .= "$_-opt ".$_POST{$tmp}."\n";
				}
			}
			print FILE "#MANUAL_TESTS:    $manual_present\n";
			print FILE "#AUTOMATIC_TESTS: $automatic_present\n";
			print FILE $test_suites;

			close(FILE);
			list_dir();
		}
		else {
			$error_text = "Cannot open file for writing ($!):<br />$full_save_name";
		}
	}
}
elsif ($_GET{"action"} eq "load_profile") {                 # Load profile
	my %profile_data = (
		"tester_name" => "",
		"organization" => "",
		"tester_email" => "",
		"send_email_report" => "0",
		"architecture" => guess_platform(`uname -m`),
		"use_internet" => "1",
		"keep_packages" => "1",
		"short_scen" => "0",
		"lsb_version" => "3.2",
		"lsb_profile" => "desktop",
		"verbose_level" => "1",
		"comments" => ""
	);
	my %tests_options = ("cmdchk" => 3, "libchk" => 3, "desktop" => 3);
	if (!$_GET{"profile"}) {
		$error_text = "No profile name specified!";
	}
	if (!$error_text) {
		my $full_save_name = $SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"};
		if (open(FILE, $full_save_name)) {
			my @comments = ();
			while (<FILE>) {
				s/[\r\n]//g;
				if (/^CERTIFY:\s+(.*)$/) {
				}
				elsif (/^NAME:\s+(.*)$/) {
					$profile_data{"tester_name"} = $1;
				}
				elsif (/^ORGANIZATION:\s+(.*)$/) {
					$profile_data{"organization"} = $1;
				}
				elsif (/^EMAIL:\s+(.*)$/) {
					$profile_data{"tester_email"} = $1;
				}
				elsif (/^SEND_EMAIL:\s+(.*)$/) {
					$profile_data{"send_email_report"} = $1;
				}
				elsif (/^VERBOSE_LEVEL:\s+(.*)$/) {
					$profile_data{"verbose_level"} = $1;
				}
				elsif (/^KEEP_PACKAGES:\s+(.*)$/) {
					$profile_data{"keep_packages"} = $1;
				}
				elsif (/^SHORT_SCENARIOS:\s+(.*)$/) {
					$profile_data{"short_scen"} = $1;
				}
				elsif (/^LSB_VERSION:\s+(.*)$/) {
					$profile_data{"lsb_version"} = $1;
				}
				elsif (/^LSB_PROFILE:\s+(.*)$/) {
					$profile_data{"lsb_profile"} = $1;
				}
				elsif (/^ARCHITECTURE:\s+(.*)$/) {
					$profile_data{"architecture"} = $1;
				}
				elsif (/^USE_INTERNET:\s+(.*)$/) {
					$profile_data{"use_internet"} = $1;
				}
				elsif (/^COMMENTS: (.*)$/) {
					push @comments, $1;
				}
				elsif (/^[^\s#]/) {
					my ($mod, $stage) = split(/\s+/, $_, 2);
					if ($mod =~ m/^(.*)-opt$/) {
						$tests_options{$1} = $stage;
					}
					else {
						$profile_data{"modules"}->{$mod} = $stage;
					}
				}
			}
			close(FILE);
			$profile_data{"comments"} = join("\n", @comments);
			if ($profile_data{"tester_email"} eq "") {
				$profile_data{"send_email_report"} = "";
			}
			$data .= "<profile_data>\n";
			foreach my $name (qw/architecture lsb_version tester_name organization tester_email lsb_profile verbose_level comments/) {
				$data .= "<item type=\"value\"><name>$name</name><value>$profile_data{$name}</value></item>\n" if (defined($profile_data{$name}));
			}
			foreach my $name (qw/send_email_report use_internet keep_packages short_scen/) {
				$data .= "<item type=\"check\"><name>$name</name><value>".($profile_data{$name} ? "1" : "0")."</value></item>\n" if (defined($profile_data{$name}));
			}
			foreach my $mod (keys %{$profile_data{"modules"}}) {
				$data .= "<item type=\"mod\"><name>$mod</name><value>".$profile_data{"modules"}->{$mod}."</value></item>\n";
			}
			foreach my $mod (keys %tests_options) {
				$data .= "<item type=\"modopt\"><name>$mod</name><value>$tests_options{$mod}</value></item>\n";
			}
			$data .= "</profile_data>\n";
			list_dir();
		}
		else {
			$error_text = "Cannot open file for reading ($!):<br />$full_save_name";
		}
	}
}
elsif ($_GET{"action"} eq "refresh_list") {                 # Refresh list of test modules: start
	my $id = $$;
	my $child = fork();
	if (!defined($child)) {
		$error_text = "Internal error: could not fork!";
	}
	elsif ($child == 0) {
		close(STDOUT);
		system($CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list-refresh -D 1>/tmp/lsb-dtk-refresh.out.$id 2>/tmp/lsb-dtk-refresh.err.$id");
		if ($? == -1) {
	    	$res = "Failed to execute lsb-tef.pl -v0 --list-refresh -D: $!";
	    }
	    else {
	    	$res = "$?";
	    }
	    if (open(FILE, ">/tmp/lsb-dtk-refresh.res.$id")) {
	    	print FILE $res;
	    	close(FILE);
	    }
		# We do not send any reply!
	    exit 0;
	}
	else {
		select(undef, undef, undef, 0.5);
		if (-f "/tmp/lsb-dtk-refresh.out.$id") {
			# There was no error; leb-tef.pl is being run.
			$data = "<pbid>id=$id,size=0</pbid>\n";
		}
		else {
			# There was an error; /tmp/lsb-dtk-refresh.res.$id contains the error message.
			$error_text = `cat /tmp/lsb-dtk-refresh.res.$id`;
			$error_text =~ s/[\r\n]//g;
		}
	}
}
elsif ($_GET{"action"} eq "refresh_list_next") {            # Refresh list of test modules: get next portion
	if (!$_GET{"pbid"}) {
		$error_text = "Incorrect input parameters! Please, contact the authors.";
	}
	else {
		my $pbid = ($_GET{"pbid"} or "");
		my ($id, $sz) = split(/,/, $pbid);
		$id =~ s/^id=(\d+)$/$1/;
		$sz =~ s/^size=(\d+)$/$1/;

		if (-f "/tmp/lsb-dtk-refresh.res.$id") {
			my $res_code = `cat /tmp/lsb-dtk-refresh.res.$id`;
			if ($res_code) {
			    if ($res_code & 127) {
					$error_text = "Error executing lsb-tef.pl (terminated with signal ".($res_code & 127).(($res_code & 128) ? ", with coredump)" : "").")!";
			    }
			    else {
					$error_text = "lsb-tef.pl exited with value ".($res_code >> 8)."!";
			    }
		    }
		    else {
				$data = "<progress><done>1</done><total>1</total></progress>\n<modlist>\n";
				my $data_html = "";
				my $data_js = "file_size = new Array();\nmod_time = new Array();\nfile_size[''] = '0';\n";
				if (open(FILE, "/tmp/lsb-dtk-refresh.out.$id")) {
					seek(FILE, $sz, SEEK_SET);
					my %lsbv_arch = ();
					while (<FILE>) {
						s/[\r\n]//g;
						if (m/^ERROR: (.*)$/i) {
							$error_text .= $1."<br />";
						}
						elsif (m/^FILE_SIZE:/) {
							my ($sig, $name, $sz) = split(/\s+/, $_, 3);
							$data_js .= "file_size['$name'] = '$sz';\n";
						}
						elsif (m/^MODULE_TIME:/) {
							my ($sig, $name, $tm) = split(/\s+/, $_, 3);
							$name =~ s/\+/p/g;
							$data_js .= "mod_time['$name'] = '$tm';\n";
						}
						elsif (m/^(ARCHITECTURE|LSB_VERSION|LSB_PROFILE|EXPIRED):/) {
							# Do nothing
						}
						elsif (m/^[^\s#]/) {
							my ($lsb_version, $arch, $mod, $mod_name, $stage, $vers, $files) = split(/\t+/, $_, 7);
							$mod =~ s/\+/p/g;
							my @file_names = (($files eq "") ? () : split(/,\s*/, $files));
							$lsbv_arch{$lsb_version}->{$arch}->{$mod}->{"STAGES"}->{$stage}->{"VERSION"} = $vers;
							$lsbv_arch{$lsb_version}->{$arch}->{$mod}->{"STAGES"}->{$stage}->{"FILES"} = \@file_names;
							$lsbv_arch{$lsb_version}->{$arch}->{$mod}->{"DISPLAY_NAME"} = $mod_name;
						}
					}
					close(FILE);
					my %archs_list = ();
					foreach my $lsb_v (@lsb_versions_list) {
						foreach my $arch (keys %{$lsbv_arch{$lsb_v}}) {
							$archs_list{$arch} = 1;
							$lsbv_arch{$lsb_v}->{$arch}->{"manappbat"}->{"STAGES"}->{"p"}->{"VERSION"} = $lsb_v;
							$lsbv_arch{$lsb_v}->{$arch}->{"manappbat"}->{"STAGES"}->{"p"}->{"FILES"} = [];
							$lsbv_arch{$lsb_v}->{$arch}->{"manappbat"}->{"DISPLAY_NAME"} = "Application Battery";
						}
					}
					$data_html .= generate_modlist(\%lsbv_arch);
					$data_js .= generate_js_mod_cat_list(\%lsbv_arch);
					$data_js .= generate_js_check_list(\%lsbv_arch);
					$data_js .= generate_js_arch_mod(\%lsbv_arch);
					$data_js .= generate_js_mod_list(\%lsbv_arch);
					$data_js .= "arch_list = new Array('".join("', '", keys %archs_list)."');\n";
					# Cut HTML and JS data into blocks by not more than 4000 symbols.
					# Reason: in Mandriva 2008 XML parser in Firefox does not allow XML nodes longer than 4096 symbols.
					my @data_html_arr = split(/\n/, $data_html);
					my $block = "";
					my $len = 0;
					for (my $i=0; $i<scalar(@data_html_arr); $i++) {
						if (length($data_html_arr[$i]) > 4000) {
							$error_text = "Internal error: too long HTML line!<br />Please, write at <a href=\"mailto:linux\@ispras.ru\"><u>linux\@ispras.ru</u></a>.";
							last;
						}
						$block .= $data_html_arr[$i]."\n";
						$len += length($data_html_arr[$i]);
						if (($i == $#data_html_arr) or (($len + length($data_html_arr[$i + 1])) > 4000)) {
							$data .= "<html><![CDATA[$block]]></html>\n";
							$block = "";
							$len = 0;
						}
					}
					my @data_js_arr = split(/\n/, $data_js);
					$block = "";
					$len = 0;
					for (my $i=0; $i<scalar(@data_js_arr); $i++) {
						if (length($data_js_arr[$i]) > 4000) {
							$error_text = "Internal error: too long HTML line!<br />Please, write at <a href=\"mailto:linux\@ispras.ru\"><u>linux\@ispras.ru</u></a>.";
							last;
						}
						$block .= $data_js_arr[$i]."\n";
						$len += length($data_js_arr[$i]);
						if (($i == $#data_js_arr) or (($len + length($data_js_arr[$i + 1])) > 4000)) {
							$data .= "<js><![CDATA[$block]]></js>\n";
							$block = "";
							$len = 0;
						}
					}
					if (-s "/tmp/lsb-dtk-refresh.err.$id" != 0) {
						$error_text .= "There were some errors while retrieving the list of available test modules!<br />See details in the <a href=\"get.pl".$SERVER_PARAM{"APP_DATA"}."/log/dtk-server.log.8888\"><u>server log file</u></a>.<br />";
						print STDERR `cat /tmp/lsb-dtk-refresh.err.$id`;
					}
					if ($error_text ne "") {
						$error_text .= "Please, see the <a href=\"tests_help.pl#troubleshooting\"><u>Troubleshooting</u></a> Help section to find the solution.<br />If you could not find any, please, write at <a href=\"mailto:linux\@ispras.ru\"><u>linux\@ispras.ru</u></a>.";
					}
				}
				$data .= "</modlist>\n";
			}
			unlink("/tmp/lsb-dtk-refresh.out.$id");
			unlink("/tmp/lsb-dtk-refresh.err.$id");
			unlink("/tmp/lsb-dtk-refresh.res.$id");
		}
		else {
			while (-s "/tmp/lsb-dtk-refresh.out.$id" <= $sz) {
				select(undef, undef, undef, 0.5);
			}
			if (open(FILE, "/tmp/lsb-dtk-refresh.out.$id")) {
				seek(FILE, $sz, SEEK_SET);
				my $file_data = "";
				read(FILE, $file_data, (-s "/tmp/lsb-dtk-refresh.out.$id") - $sz);
				close(FILE);
				my @lines = split(/([\r\n]+)/, $file_data);
				pop(@lines) if (scalar(@lines) % 2 == 1);
				my $done;
				my $total;
				for (my $i=0; $i<scalar(@lines); $i++) {
					if ($lines[$i] =~ m/^#PROGRESS:\s*(\d+)\/(\d+)\s*$/) {
						($done, $total) = ($1, $2);
					}
					elsif ($lines[$i] !~ m/^#/) {
						last;
					}
					$sz += length($lines[$i]) + length($lines[$i + 1]);
					$i++;
				}
				if ($done) {
					$data = "<pbid>id=$id,size=$sz</pbid>\n<progress><done>$done</done><total>$total</total></progress>\n";
				}
				else {
					$data = "<pbid>id=$id,size=$sz</pbid>\n";
				}
			}
			else {
				$data = "<pbid>id=$id,size=$sz</pbid>\n";
			}
		}
	}
}
elsif ($_GET{"action"} eq "run_tests") {                    # Start the tests
	if (!$_GET{"profile"}) {
		$error_text = "Incorrect call parameters!";
	}
	else {
		my $running = 0;
		if (-f "/tmp/tests_output.8888") {
			`head -n 1 /tmp/tests_output.8888 2>/dev/null` =~ m/PID:\s*(\d+),\s*CHLD_PID:\s*(\d+)(\s*\(cert\))?/i;
			$running = is_process_running($1);
		}
		if ($running) {
			$error_text = "Tests are already running.<br />You cannot run another instance before they are finished.<br />Start watching the current run?<br /><br /><a href=\"#\" onclick=\"javascript:startRefresh(); return false;\">Yes</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"#\" onclick=\"javascript:err_area.style.display='none'; return false;\">No</a>";
		}
		else {
			my $profile_path = $SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"};
			my $child = fork();
			if (!defined($child)) {
				$error_text = "Cannot create child process!";
			}
			elsif ($child == 0) {
				close(STDOUT);
				exec($CONFIG{"TESTS_DIR"}."/run_tests.pl \"".$CONFIG{"TESTS_DIR"}."\" \"".$CONFIG{"RESULTS_DIR"}."\" \"$profile_path\"");
			}
			else {
				$data = "<started>1</started>\n";
				my $cert = "0";
				if (open(FILE, $profile_path)) {
					while (<FILE>) {
						if (m/^CERTIFY:\s*(\d+)$/) {
							$cert = ($1 ? "1" : "0");
							last;
						}
					}
					close(FILE);
				}
				$data .= "<cert>$cert</cert>\n";
			}
		}
	}
}
elsif ($_GET{"action"} eq "get_tests_output") {             # Get next portion of test log output
	my $start = ($_GET{"start"} or 0);
	if (($start != 0) and !$_GET{"fid"}) {
		$error_text = "Incorrect call parameters!";
	}
	else {
		my $running;
		my $log_file = "/tmp/tests_output.8888";
		my $fid = ($_GET{"fid"} or 0);
		if ((-f $log_file) or (-f "/tmp/tests_status.8888")) {      # Tests are running or finished
			my $res_code;
			my $res_dir;
			if (-f "/tmp/tests_status.8888") {
				if (open(FILE_STAT, "/tmp/tests_status.8888")) {
					flock(FILE_STAT, LOCK_SH);
					my $file_data = "";
					read(FILE_STAT, $file_data, -s "/tmp/tests_status.8888");
					flock(FILE_STAT, LOCK_UN);
					close(FILE_STAT);
					($res_code, $res_dir) = split(/\n/, $file_data);
				}
			}
			if (-f $log_file) {                             # Tests are either running or killed
				`head -n 1 $log_file 2>/dev/null` =~ m/PID:\s*(\d+),\s*CHLD_PID:\s*(\d+)(\s*\(cert\))?/i;
				$running = is_process_running($1);
			}
			else {                                          # Tests have been finished, look for new log file location
				$running = 0;
				if ($res_dir) {
					$log_file = $CONFIG{"RESULTS_DIR"}."/".$res_dir."/test_logs/tests_output.8888";
				}
				if (!-f $log_file) {
					$log_file = "/tmp/tests_output.8888.old";
				}
			}
			if (-f $log_file) {
				my $fid_current = get_file_id($log_file);
				$fid = $fid_current if (!$fid);
				if ($fid eq $fid_current) {                 # The log file is the same we started with
					`head -n 1 $log_file 2>/dev/null` =~ m/PID:\s*(\d+),\s*CHLD_PID:\s*(\d+)(\s*\(cert\))?/i;
					$data .= "<fid>$fid</fid>\n<cert>".($3 ? "1" : "0")."</cert>\n";
					if (open(FILE, $log_file)) {
						seek(FILE, $start, SEEK_SET);
						my $file_data = "";
						my $sz = read(FILE, $file_data, (-s $log_file) - $start);
						close(FILE);
						if ($start == 0) {
							$start = index($file_data, "\n") + 1;
							$file_data = substr($file_data, $start);
						}
						$start += process_output($file_data);
						my $len = length($file_data);
						my $b_pos = 0;
						while ($b_pos < $len) {
							my $b_len = $len - $b_pos;
							if ($b_len > 4000) {
								$b_len = 4000;
							}
							my $block = substr($file_data, $b_pos, $b_len);
							$b_pos += $b_len;
							$data .= "<output><![CDATA[$block]]></output>\n";
						}
						if ($running) {
							$data .= "<size>$start</size>\n";
						}
						else {
							my $status = "";
							if (defined($res_code)) {
								if ($res_code == 0) {       # Test run finished successfully, get the tests result
									$status = "<font color=\"#ff7f00\">Warning: Could not obtain tests result!</font>";
									if ($res_dir) {
										if (open(FILE_INFO, $CONFIG{"RESULTS_DIR"}."/$res_dir/INFO")) {
											while (<FILE_INFO>) {
												s/[\r\n]//g;
												if (m/^Verdict:\s*(.*)/i) {
													if ($1 eq "Success") {
														$status = "Status: <font color=\"green\">Success.</font>";
														$data .= "<redirect>$res_dir</redirect>\n";
													}
													elsif ($1 eq "Failed") {
														$status = "Status: <font color=\"red\">Failed.</font>";
														$data .= "<redirect>$res_dir</redirect>\n";
													}
												}
											}
											close(FILE_INFO);
										}
									}
								}
							    elsif ($res_code & 127) {
									$status = "Status: <font color=\"red\">Process killed by signal ".($res_code & 127).(($res_code & 128) ? " (with coredump)" : "")."!</font>";
							    }
							    else {
									$status = "Status: <font color=\"red\">Error (returned code: ".($res_code >> 8).")!</font>";
							    }
						    }
						    else {
						    	$status = "<font color=\"#ff7f00\">Warning: Could not obtain return code!</font>";
						    }
						    $data .= "<status><![CDATA[$status]]></status>\n";
						}
					}
					else {
						$error_text = "Could not open output log &lsquo;$log_file&rsquo;!<br />$!";
					}
				}
				else {                                      # The new test run seems to have been running
					$error_text = "The requested log file no longer exists.";
				}
			}
			else {
				$error_text = "No output log found!";
			}
		}
		else {
			$error_text = "Output log and status file were not found!";
		}
	}
}
elsif ($_GET{"action"} eq "stop_tests") {                   # Stop the tests
	if (!$_GET{"tree"}) {
		$error_text = "Incorrect call parameters!";
	}
	else {
		my $PID;
		if ($_GET{"tree"} eq "tests") {
			`head -n 1 /tmp/tests_output.8888 2>/dev/null` =~ m/CHLD_PID:\s*(\d+)/i;
			$PID = $1;
		}
		elsif ($_GET{"tree"} eq "server") {
			`head -n 1 $SERVER_PARAM{APP_DATA}/log/dtk-server.log.8888 2>/dev/null` =~ m/PID:\s*(\d+)/i;
			$PID = $1;
		}
		else {
			$error_text = "Incorrect call parameters!";
		}
		if (!$error_text) {
			system($CONFIG{"TESTS_DIR"}."/stop_tests.pl $PID ".$SERVER_PARAM{"APP_DATA"}."/log/dtk-server.log.8888 >/tmp/dtk-stop-tests.out.8888 2>&1");
			if ($? == 0) {
				$data = "<stopped id=\"".$_GET{"tree"}."\">1</stopped>\n";
			}
			else {
				$error_text = `cat /tmp/dtk-stop-tests.out.8888`;
			}
			unlink("/tmp/dtk-stop-tests.out.8888");
		}
	}
}
else {
	$error_text = "Incorrect call parameters!";
}

send_reply();
