#!/usr/bin/perl -w

# LSB Test Execution Framework
# Manual Application Battery Tests Page Module (tests_appbat.pl)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

my %tests_length = ();          # Number of questions in each of the tests available
my %tests_result = ();          # Results of the tests execution stored in the journal files (if present)
my %result_display = (-1 => '<font color="blue">Not finished</font>', 0 => '<font color="red">Failed</font>', 1 => '<font color="green">Passed</font>');

my @test_cases = ();            # Questions in the test chosen
my @test_cases_expected = ();   # Expected results in the test chosen
my @test_cases_answers = ();    # Results stored in the journal for the test chosen
my $test_comments = "";         # Comments for the test chosen

my $lsb_version = (`cat $SERVER_PARAM{"APP_DATA"}/results/$_GET{"test_run"}/host_info/lsb_version 2>/dev/null` or "3.2");
$lsb_version =~ s/[\r\n]//g;

my $idx;
my $error = "";

# Check if the necessary arguments are specified.
if (!$_GET{"test_run"}) {
	$error = "Test run is not specified!";
}
elsif ($_POST{"finish_test"} and !$_GET{"test"}) {
	$error = "Test name is not specified when saving!";
}

# Read list of the tests available.
if (!$error) {
	if (opendir(APPBAT_DIR, $SERVER_PARAM{"DOCUMENT_ROOT"}."/app_battery")) {
		foreach (grep /^$lsb_version/, readdir(APPBAT_DIR)) {
			s/^$lsb_version-(.*)\.inc/$1/;
			$tests_length{$_} = -1;
			$tests_result{$_} = -1;
		}
		closedir(APPBAT_DIR);
		if (scalar(keys %tests_length) == 0) {
			$error = "No suitable manual tests were found!";
		}
	}
	else {
		$error = "Cannot read the directory contents:<br />\n<font color=\"blue\"><code>".$SERVER_PARAM{"DOCUMENT_ROOT"}."/app_battery</code></font><br /><br />\n$!";
	}
}

# If no test specified, open the first one.
if (!$error and !$_GET{"test"}) {
	my @tmp = sort keys %tests_length;
	$_GET{"test"} = $tmp[0];
}

# Read the specific test data if the test name is specified.
if (!$error) {
	if (open(TEST_DATA, $SERVER_PARAM{"DOCUMENT_ROOT"}."/app_battery/$lsb_version-$_GET{test}.inc")) {
		$idx = 0;
		while(<TEST_DATA>) {
			if (m/^\* /) {
				$idx++;
				s/^\* //;
				s/[\r\n]//g;
				$test_cases[$idx] = $_;
				$test_cases_expected[$idx] = "";
			}
			else {
				$test_cases_expected[$idx] .= $_;
			}
		}
		close(TEST_DATA);
		foreach (1..$idx) {
			$test_cases_expected[$_] =~ s/\n$// if (($test_cases_expected[$_] =~ s/\r\n$//) == 0);
		}
	}
	else {
		$error = "Cannot read the test data file:<br />\n<font color=\"blue\"><code>".$SERVER_PARAM{"DOCUMENT_ROOT"}."/app_battery/$lsb_version-$_GET{test}.inc</code></font><br /><br />\n$!";
	}
}

# If POST-query "Finish test", save the journal and reload page.
if (!$error and $_POST{"finish_test"}) {
	my @tests = sort keys %tests_length;
	my $i;
	for ($i=0; $i<=$#tests; $i++) {
		last if ($tests[$i] eq $_GET{"test"});
	}
	$i++ if ($i < $#tests);
	my $next_test = $tests[$i];
	if (!open(TEST_JOURNAL, ">".$SERVER_PARAM{"APP_DATA"}."/results/$_GET{test_run}/test_journals/$_GET{test}.simplejournal")) {
		$error = "Cannot write the test journal file:<br />\n<font color=\"blue\"><code>".$SERVER_PARAM{"APP_DATA"}."/results/$_GET{test_run}/test_journals/$_GET{test}.simplejournal</code></font><br /><br />\n$!";
	}
	else {
		$idx = 1;
		while ($_POST{"Test".$idx}) {
			$test_cases_expected[$idx] =~ s/\r\n/<br \/>/g;
			$test_cases_expected[$idx] =~ s/\n/<br \/>/g;
			print TEST_JOURNAL "Q$idx: ".(($_POST{"Test".$idx} eq "Yes") ? "OK: " : "failed: ").$test_cases[$idx]."<br />".$test_cases_expected[$idx]."\n";
			$idx++;
		}
		my $comments = $_POST{"Deviations"};
		$comments =~ s/\r\n/<br \/>/g;
		$comments =~ s/\n/<br \/>/g;
		print TEST_JOURNAL "Summary: ".(($_POST{"Problems"} eq "Yes") ? "failed" : "OK").": ".$comments;
		close(TEST_JOURNAL);
	}
	if (!$error) {
		print "HTTP/1.0 302 Moved\nLocation: tests_appbat.pl?test_run=$_GET{test_run}&test=$next_test\n\n";
		exit;
	}
}

# Print header.
print "HTTP/1.0 200 OK\n";
print "Content-type: text/html\n\n";

print_header("Manual Application Battery Tests", "manappbat");
print "<h1>Manual Application Battery Tests</h1>\n";

# Count number of questions in each test.
if (!$error) {
	my @fails = ();
	foreach my $test (keys %tests_length) {
		if ($test eq $_GET{"test"}) {
			$tests_length{$test} = scalar(@test_cases) - 1;
		}
		else {
			if (open(TEST_DATA, $SERVER_PARAM{"DOCUMENT_ROOT"}."/app_battery/$lsb_version-$test.inc")) {
				$idx = 0;
				while(<TEST_DATA>) {
					$idx++ if (m/^\* /);
				}
				close(TEST_DATA);
				$tests_length{$test} = $idx;
			}
			else {
				push @fails, "<font color=\"blue\"><code>".$SERVER_PARAM{"DOCUMENT_ROOT"}."/app_battery/$lsb_version-$test.inc</code></font>: $!";
			}
		}
	}
	if (scalar(@fails) > 0) {
		$error = "Cannot read one or more test data files:<br />\n".join("<br />\n", @fails)."\n";
	}
}

if (!$error) {
	# Read journal files for the tests already performed and store the results.
	my $allow_finish = 1;
	foreach my $test (keys %tests_length) {
		if (open(TEST_JOURNAL, $SERVER_PARAM{"APP_DATA"}."/results/$_GET{test_run}/test_journals/$test.simplejournal")) {
			my %numbers = ();
			$tests_result{$test} = 1;                   # First, suppose all is OK
			while(<TEST_JOURNAL>) {
				if (m/^Q(\d+): (OK|failed)\b/) {
					$numbers{$1} = 1;
					if ($2 eq "failed") {
						$tests_result{$test} = 0;       # Not all is OK
					}
					$test_cases_answers[$1] = (($2 eq "OK") ? "yes" : "no") if ($test eq $_GET{"test"});
				}
				elsif (m/^Summary: (OK|failed)(: (.*))?$/) {
					$numbers{0} = 1;
					if ($1 eq "failed") {
						$tests_result{$test} = 0;       # Not all is OK
					}
					if ($test eq $_GET{"test"}) {
						$test_cases_answers[0] = (($1 eq "OK") ? "no" : "yes");
						if ($3) {
							$test_comments = $3;
							$test_comments =~ s/<br \/>/\n/g;
						}
					}
				}
			}
			close(TEST_JOURNAL);
			foreach (0..$tests_length{$test}) {
				if (!$numbers{$_}) {
					$tests_result{$test} = -1;          # Not all questions are answered
					last;
				}
			}
		}
		$allow_finish = 0 if ($tests_result{$test} == -1);
	}

	# Output the left pane (list of the tests).
	print <<DATA;
<table border="0" cellpadding="0" cellspacing="0">
  <tr>
    <td style="width: 21em; vertical-align: top;">
      Please, perform the following Application Battery tests to finish with the certification:<br /><br />
      <table class="main" style="width: 21em;">
DATA
	my $finish_button = ($allow_finish ?
		"<input type=\"button\" name=\"finish_appbat\" value=\"Finish\" style=\"width: 6em;\" onclick=\"javascript:location='tests_results.pl?generate=$_GET{test_run}';\" />" :
		"<input type=\"button\" name=\"finish_appbat\" value=\"Finish\" style=\"width: 6em;\" disabled=\"disabled\" /><br />You need to finish all the tests to continue.");
	foreach (sort keys %tests_result) {
		print "        <tr>\n          <td style=\"width: 6.3em;\"><a href=\"tests_appbat.pl?test_run=$_GET{test_run}&amp;test=$_\">".(($_ eq $_GET{"test"}) ? "<b>$_</b>" : $_)."</a></td>\n          <td style=\"width: 12em;\">Status: <b>".$result_display{$tests_result{$_}}."</b></td>\n        </tr>\n";
	}
	print <<DATA;
      </table>
      <br /><br />
      $finish_button
    </td>
    <td><img src="images/environment-spacer.gif" width="30" height="1" border="0" alt="" /></td>
    <td width="100%" style="border: solid 1px gray; padding: 10px;" valign="top">
DATA

	# Output the right pane (test chosen).
	print "      <form method=\"post\" action=\"tests_appbat.pl?test_run=$_GET{test_run}&amp;test=$_GET{test}\" name=\"ab_test\">\n";
	
	print <<DATA;
<h2>Functional Verification Test: $_GET{test}</h2>

<table>
<thead>
<tr>
<th>&nbsp;</th>
<th>Expected&nbsp;result&nbsp;&nbsp;<br/>achieved?</th>
<th>Test case scenerio</th>
</tr>
</thead>
<tbody>
DATA
	my $sel_yes;
	my $sel_no;
	foreach (1..$#test_cases) {
		$sel_yes = (($test_cases_answers[$_] and ($test_cases_answers[$_] eq "yes")) ? " checked=\"checked\"" : "");
		$sel_no = (($test_cases_answers[$_] and ($test_cases_answers[$_] eq "no")) ? " checked=\"checked\"" : "");
		print <<DATA;
<tr>
<td valign="top">$_.</td>
<td valign="top">
<span id="Test$_\_bg" style="white-space: nowrap;"><label><input type="radio" name="Test$_" id="Test$_\_yes" value="Yes" onclick="javascript:clear_bg('Test$_');"$sel_yes />Yes</label> <label><input type="radio" name="Test$_" id="Test$_\_no" value="No" onclick="javascript:clear_bg('Test$_');"$sel_no />No</label></span>
</td>
<td>
$test_cases[$_]<br />
$test_cases_expected[$_]
</td>
</tr>
DATA
	}
	print <<DATA;
</tbody>
</table>

DATA
	$sel_yes = (($test_cases_answers[0] and ($test_cases_answers[0] eq "yes")) ? " checked=\"checked\"" : "");
	$sel_no = (($test_cases_answers[0] and ($test_cases_answers[0] eq "no")) ? " checked=\"checked\"" : "");
	print <<DATA;
<h2>Results</h2>

<table>
<tr><td valign="top">1.</td>

<td valign="top">
<span id="Problems_bg" style="white-space: nowrap;"><label><input type="radio" name="Problems" id="Problems_yes" value="Yes" onclick="javascript:clear_bg('Problems');"$sel_yes />Yes</label> <label><input type="radio" name="Problems" id="Problems_no" value="No" onclick="javascript:clear_bg('Problems');"$sel_no />No</label></span>
</td>

<td>Were there any deviations in the install, setup, or test described above?
<br />If yes, please describe.
<br /><textarea name="Deviations" rows="3" cols="60">$test_comments</textarea></td>
</tr>
</table>

        <br />
        <input type="submit" name="finish_test" id="finish_test" value="Finish test" style="width: 6em;" onclick="javascript:return check_finish();" />&nbsp;&nbsp;&nbsp;<input type="reset" name="reset_test" value="Reset test" style="width: 6em;" onclick="javascript:return check_reset();" />
      </form>
    </td>
  </tr>
</table>
DATA

	my $num_elems = $#test_cases;

	print <<DATA;
<script language="javascript" type="text/javascript">
// <![CDATA[
function check_finish() {
	var i;
	var elem_yes;
	var elem_no;
	var elem;
	var unchecked = 0;
	for (i=1; i<=$num_elems; ++i) {
		elem_yes = document.getElementById("Test" + i + "_yes");
		elem_no = document.getElementById("Test" + i + "_no");
		if (elem_yes && elem_no) {
			if (!elem_yes.checked && !elem_no.checked) {
				++unchecked;
				elem = document.getElementById("Test" + i + "_bg");
				if (elem) {
					elem.style.backgroundColor = "#ff7f7f";
				}
			}
		}
		else {
			++unchecked;
		}
	}
	elem_yes = document.getElementById("Problems_yes");
	elem_no = document.getElementById("Problems_no");
	if (elem_yes && elem_no) {
		if (!elem_yes.checked && !elem_no.checked) {
			++unchecked;
			elem = document.getElementById("Problems_bg");
			if (elem) {
				elem.style.backgroundColor = "#ff7f7f";
			}
		}
	}
	else {
		++unchecked;
	}
	if (unchecked > 0) {
		var single_plural = ((unchecked == 1) ? "is 1 question" : "are " + unchecked + " questions");
		alert("Please, answer all the questions of the test!\\nThere " + single_plural + " remained unanswered.");
		return false;
	}
	else {
		return true;
	}
}

function clear_bg(name) {
	elem = document.getElementById(name + "_bg");
	if (elem) {
		elem.style.backgroundColor = "";
	}
}

function check_reset() {
	if (confirm('Are you sure you wish to clear your answers?')) {
		var i;
		for (i=1; i<=$num_elems; ++i) {
			clear_bg("Test" + i);
		}
		clear_bg("Problems");
		return true;
	}
	else {
		return false;
	}
}
// ]]>
</script>
DATA
}
else {
	print "<h3><font color=\"red\" size=\"+1\">Error!</font></h3>\n$error\n";
}

print_footer();
