#!/usr/bin/perl -w

# LSB Test Execution Framework
# Configuration Save Module (tests_conf_save.pl)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use Fcntl ':flock';

sub is_process_finished($) {
	my $PID = $_[0];
	my $test_line = `ps -p $PID -o s --no-heading`;
	$test_line =~ s/[\r\n]//g;
	return ($test_line !~ m/^[^Zz]/);
}

my $conf_page = ($_POST{"certify"} ? "tests_cert.pl" : "tests_conf.pl");
if ($_GET{"goto"}) {
	$conf_page = $_GET{"goto"};
	$conf_page =~ s/%23/#/;
	$_POST{"form_action"} = "";
}
my $session_profile = ($_COOKIE{"session_id"} ? "~session.".$SERVER_PARAM{"PEER_IP"}.".".$_COOKIE{"session_id"} : "");

my $error;
my $new_url = "";
my $save_profile_name = "";
my @cookies = ();

if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
	system("mkdir -p ".$SERVER_PARAM{"APP_DATA"}."/profiles");
	if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
		$error = "";
	}
}

if (!defined($error)) {
	if ((!$_POST{"form_action"} or ($_POST{"form_action"} eq "")) and (!$_GET{"goto"})) {
		$error = "Script action not specified!";
	}
	elsif ($_POST{"form_action"} eq "run_tests") {
		my $tests_exec = 0;
		if (-f "/tmp/tests_output.8888") {
			if (open(FILE, "< /tmp/tests_output.8888")) {
				my $PID = <FILE>;
				$PID =~ m/PID: (\d+),.*/;
				$PID = $1;
				if (is_process_finished($PID) and flock(FILE, LOCK_EX | LOCK_NB)) {
#					unlink("/tmp/tests_output.8888");
					rename("/tmp/tests_output.8888", "/tmp/tests_output.8888.old");
					flock(FILE, LOCK_UN);
					$tests_exec = 1;
				}
				close(FILE);
			}
		}
		else {
			$tests_exec = 1;
		}

		if (!$tests_exec) {
			$save_profile_name = $session_profile;
			$error = "Tests are already running!<br />You cannot run another instance before they are finished.<br />You can watch or stop the running tests on the <a href=\"tests_exec.pl\"><u>Execution</u></a> page.";
		}
		else {
			if ($session_profile eq "") {
				$error = "Cannot retrieve session ID! Please, allow cookies in your browser.";
			}
			else {
				$new_url = "tests_exec.pl";
				push @cookies, "profile=$session_profile";
				$save_profile_name = $session_profile;
			}
		}
	}
	elsif ($_GET{"goto"}) {
		$save_profile_name = $session_profile;
		$new_url = $conf_page;
	}
}

if (!defined($error)) {
	if ($save_profile_name ne "") {
		my $full_save_name = $SERVER_PARAM{"APP_DATA"}."/profiles/$save_profile_name";
		if (open(FILE, ">$full_save_name")) {
			my $architecture = ($_POST{"architecture"} or "ia32");
			my $lsb_version = ($_POST{"lsb_version"} or "3.2");
			my $lsb_profile = ($_POST{"lsb_profile"} or "desktop");
			print FILE "CERTIFY:         ".$_POST{"certify"}."\n"      if ($_POST{"certify"});
			print FILE "NAME:            ".$_POST{"tester_name"}."\n"  if ($_POST{"tester_name"});
			print FILE "ORGANIZATION:    ".$_POST{"organization"}."\n" if ($_POST{"organization"});
			if ($_POST{"tester_email"}) {
				print FILE "EMAIL:           ".$_POST{"tester_email"}."\n";
				print FILE "SEND_EMAIL:      ".(defined($_POST{"send_email_report"}) ? "1" : "0")."\n";
			}

			print FILE "VERBOSE_LEVEL:   ".($_POST{"verbose_level"} ? $_POST{"verbose_level"} : "1")."\n";
			print FILE "KEEP_PACKAGES:   ".(defined($_POST{"keep_packages"}) ? "1" : "0")."\n";
			print FILE "SHORT_SCENARIOS: ".(defined($_POST{"short_scen"}) ? "1" : "0")."\n";
			print FILE "ARCHITECTURE:    $architecture\n";
			print FILE "USE_INTERNET:    ".(defined($_POST{"use_internet"}) ? "1" : "0")."\n";
			print FILE "LSB_VERSION:     $lsb_version\n";
			print FILE "LSB_PROFILE:     $lsb_profile\n";

			if ($_POST{"comments"}) {
				$_POST{"comments"} =~ s/\r\n/\n/g;
				$_POST{"comments"} =~ s/&/&amp;/g;
				$_POST{"comments"} =~ s/</&lt;/g;
				$_POST{"comments"} =~ s/>/&gt;/g;
				foreach (split(/\n/, $_POST{"comments"})) {
					print FILE "COMMENTS: $_\n";
				}
			}

			my $manual_present = 0;
			my $automatic_present = 0;
			my $test_suites = '';
			foreach (sort grep /mod-$architecture-$lsb_version-/, keys %_POST) {
				my $tmp = $_;
				$tmp =~ s/^mod-/modstage-/;
				s/^mod-$architecture-$lsb_version-//;
				$test_suites .= "$_ ".$_POST{$tmp}."\n";
				if ($_ eq 'manappbat') {
					$manual_present = 1;
				}
				else {
					$automatic_present = 1;
				}
				$tmp =~ s/^modstage-/modopt-/;
				if ($_POST{$tmp}) {
					$test_suites .= "$_-opt ".$_POST{$tmp}."\n";
				}
			}
			print FILE "#MANUAL_TESTS:    $manual_present\n";
			print FILE "#AUTOMATIC_TESTS: $automatic_present\n";
			print FILE $test_suites;

			close(FILE);

			if ($_POST{"form_action"} eq "run_tests") {
				if (open(CHK_FILE, $CONFIG{"TESTS_DIR"}."/lsb-tef.pl --check-only -f \"$full_save_name\" |")) {
					$error = "";
					while(<CHK_FILE>) {
						s/[\r\n]//g;
						if (m/^ERROR: (.*)/i) {
							$error .= ($error ? "<br />" : "").$1;
						}
					}
					close(CHK_FILE);
					undef $error if ($error eq "");
					if (defined($error)) {
						$error .= "<br />Please, see the <a href=\"tests_help.pl#troubleshooting\"><u>Troubleshooting</u></a> Help section to find the solution.";
						$error .= "<br />If you could not find any, please, write at <a href=\"mailto:linux\@ispras.ru\"><u>linux\@ispras.ru</u></a>.";
					}
				}
				else {
					$error = "Cannot run lsb-tef.pl to check the requirements!<br />$!";
				}
			}
		}
		else {
			$error = "Cannot open file $save_profile_name for writing!<br />$!";
		}
	}
}

if (defined($error)) {
	escape($error);
	print "HTTP/1.0 302 Moved\n";
	print "Set-Cookie: last_error_text=$error; path=/;\n";
	print "Location: $conf_page\n\n";
}
else {
	print "HTTP/1.0 302 Moved\n";
	print "Location: $new_url\n";
	foreach (@cookies) {
		print "Set-Cookie: $_; path=/;\n";
	}
	print "\n";
}
